package jesse.keeblarcraft.BankMgr;

import java.util.HashMap;
import java.util.Map.Entry;

import jesse.CommonServerUtils;
import jesse.keeblarcraft.ConfigMgr.ConfigManager;
import jesse.keeblarcraft.Utils.ChatUtil;
import jesse.keeblarcraft.Utils.ChatUtil.CONSOLE_COLOR;
import jesse.keeblarcraft.Utils.CustomExceptions.FILE_WRITE_EXCEPTION;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.PlayerManager;
import net.minecraft.server.network.ServerPlayerEntity;

public class BankManager {
    BankManagerFile bankInfo = new BankManagerFile();
    ConfigManager config = new ConfigManager(); // for read and write privs
    CommonServerUtils commonServerUtils = new CommonServerUtils();

    public BankManager(String uuid) {
        Boolean existingFile = false;
        try {
            bankInfo = config.GetJsonObjectFromFile("bank/" + uuid + ".json", BankManagerFile.class);
            existingFile = true;
        } catch (Exception e) {
            // Do nothing. This means the file does not exist
        }

        // In the event the above code failed out, this means a new file has to be
        // created for the player's uuid
        if (!existingFile) {
            System.out.println(ChatUtil.ColoredString("Trying to create new file", CONSOLE_COLOR.BLUE));
            try {
                FlashConfig(bankInfo.uuid);
            } catch (Exception e) {
                System.out.println(ChatUtil.ColoredString("Could not write to file", CONSOLE_COLOR.RED));
            }
        } else {
            System.out.println(ChatUtil.ColoredString("Moving on", CONSOLE_COLOR.BLUE));
        }

        if ("".equals(bankInfo.uuid)) {
            System.out.println(ChatUtil.ColoredString("Assigning new config file for this uuid. No previous existing",
                    CONSOLE_COLOR.BLUE));
            bankInfo.uuid = uuid;
        }
    }

    //this class is the structure for the hashmap in BankManagerFile
    private class BankManagerMetaData {
        public BankManagerMetaData(long money, String reason, long payment, String otherParty, Integer time) {
            this.balance = money;
            this.reason = reason;
            this.payment = payment;
            this.otherParty = otherParty;
            this.time = time;
        }

        long balance = 0;
        String reason; //not sure why my compiler is saying unused
        long payment;
        String otherParty;
        Integer time;
    }

    // This is the general bank account of the read-in config for the player uuid ||| the class that gets converted into a json for the players file
    public class BankManagerFile {
        // Players uuid is the name of the file
        String uuid;

        // Contents of file
        /*
         * Example:
         * player_uuid_here:
         * {
         * "1":
         * {
         * "balance": "10";
         * "reason": "tax evasion";
         * "payment": $-44
         * "other party": "jt";
         * "time": "30";
         * }
         * "2":
         * {
         * Etc.
         * }
         * }
         */
        public HashMap<String, BankManagerMetaData> bank = new HashMap<String, BankManagerMetaData>();
    }

    public long GetBalance() {
        long ret = 0;
        for (Entry<String, BankManagerMetaData> entry : bankInfo.bank.entrySet()) {
            ret = entry.getValue().balance;
        }
        return ret;
    }

    public void SetBalance(long payment, String reason, ServerPlayerEntity otherParty) {
        bankInfo.uuid = otherParty.getUuidAsString();
        bankInfo.bank.put(bankInfo.uuid, new BankManagerMetaData(payment, reason, payment, "SERVER", 0));
        FlashConfig(bankInfo.uuid);
    }

    public void AddMoney(ServerPlayerEntity otherParty, long payment, ServerPlayerEntity player, String reason) {
        bankInfo.uuid = otherParty.getUuidAsString();
        bankInfo.bank = DoesBankAccountExist(otherParty);
        if (bankInfo.bank.size() > 0) {
            for (Entry<String, BankManagerMetaData> entry : bankInfo.bank.entrySet()) {
                entry.getValue().balance += payment;
                entry.getValue().reason = "SERVER: " + reason;
                entry.getValue().payment = payment;
                entry.getValue().otherParty = "SERVER";
                entry.getValue().time = 0;
            }
        } else {
            bankInfo.bank.put(bankInfo.uuid, new BankManagerMetaData(payment, reason, payment, "SERVER", 0));
        }

        FlashConfig(otherParty.getUuidAsString());
    }

    public void SubtractBalance(String reason, long payment, ServerPlayerEntity otherParty) {
        bankInfo.uuid = otherParty.getUuidAsString();
        bankInfo.bank = DoesBankAccountExist(otherParty);
            for (Entry<String, BankManagerMetaData> entry : bankInfo.bank.entrySet()) {
                entry.getValue().balance -= payment;//not working?
                entry.getValue().reason = "SERVER: " + reason;
                entry.getValue().payment = payment;
                entry.getValue().otherParty = "SERVER";
                entry.getValue().time = 0;
            }
        FlashConfig(bankInfo.uuid);
    }

    public void Wire(ServerPlayerEntity otherParty, long payment, String reason, ServerPlayerEntity sender) {

        BankManagerFile sendersBankInfo = new BankManagerFile();
        BankManagerFile otherPartyBankInfo = new BankManagerFile();
        sendersBankInfo.uuid = sender.getUuidAsString();
        otherPartyBankInfo.uuid = otherParty.getUuidAsString();
        sendersBankInfo.bank = DoesBankAccountExist(sender);
        otherPartyBankInfo.bank = DoesBankAccountExist(otherParty);


        //check to make sure they can afford to send the money
        for (Entry<String, BankManagerMetaData> entry : sendersBankInfo.bank.entrySet()) {
            if (entry.getValue().balance > payment) {
                entry.getValue().balance -= payment;
                entry.getValue().reason = reason;
                entry.getValue().payment = payment;
                entry.getValue().otherParty = otherParty.getUuidAsString();
                entry.getValue().time = 0;
            } else {
            System.out.println(ChatUtil.ColoredString("You ain't the goverment: " + otherParty, CONSOLE_COLOR.RED));
            return;
            }
        }
        for (Entry<String, BankManagerMetaData> entry : otherPartyBankInfo.bank.entrySet()) {
            entry.getValue().balance += payment;
            entry.getValue().reason = reason;
            entry.getValue().payment = payment;
            entry.getValue().otherParty = otherParty.getUuidAsString();
            entry.getValue().time = 0;
        }
            //needs to be inside the player check flash config has a write check
            FlashConfig(otherPartyBankInfo.uuid);
            FlashConfig(sendersBankInfo.uuid);
    }

    public HashMap<String, BankManagerMetaData> DoesBankAccountExist(ServerPlayerEntity otherParty) {
        Boolean existingFile = false;
        try {
            bankInfo.uuid = otherParty.getUuidAsString();
            bankInfo = config.GetJsonObjectFromFile("bank/" + otherParty.getUuidAsString() + ".json", BankManagerFile.class);
            existingFile = true;
        } catch (Exception e) {
            // Do nothing. This means the file does not exist
        }
        if (!existingFile) {
            System.out.println("Procced no account exisiting");
            BankManagerFile newBankInfo = new BankManagerFile();
            BankManagerMetaData newBank = new BankManagerMetaData(0, "Account Created", 0, "Server", 0);
            newBankInfo.uuid = otherParty.getUuidAsString();
            newBankInfo.bank.put(newBankInfo.uuid, newBank);
            try {
                config.WriteToJsonFile("bank/" + newBankInfo.uuid + ".json", newBankInfo);
            } catch (FILE_WRITE_EXCEPTION e) {
                System.out.println(ChatUtil.ColoredString("Could not flash notes configuration file", CONSOLE_COLOR.RED));
                return newBankInfo.bank;
            }
            return newBankInfo.bank;
        } else {
            try {
                bankInfo = config.GetJsonObjectFromFile("bank/" + bankInfo.uuid + ".json", BankManagerFile.class);
            } catch (Exception e) {
                // Do nothing. This means the file does not exist
                System.out.println("How did this fail Function: DoesBankAccountExist try catch statement -first if failed incorrectly");
            }
            return bankInfo.bank;
        }
    }

    public void FlashConfig(String uuid) {
        try {
            config.WriteToJsonFile("bank/" + uuid + ".json", bankInfo);
        } catch (FILE_WRITE_EXCEPTION e) {
            System.out.println(ChatUtil.ColoredString("Could not flash notes configuration file", CONSOLE_COLOR.RED));
        }
    }
}
