package jesse.keeblarcraft.Commands;

import com.mojang.brigadier.arguments.IntegerArgumentType;
import com.mojang.brigadier.arguments.LongArgumentType;
import com.mojang.brigadier.arguments.StringArgumentType;
import com.mojang.brigadier.context.CommandContext;

import jesse.keeblarcraft.BankMgr.BankManager;
import jesse.keeblarcraft.ConfigMgr.ConfigManager;
import net.minecraft.command.argument.EntityArgumentType;
import net.minecraft.server.command.CommandManager;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.Text;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;

public class BankCommands {
    ConfigManager config = new ConfigManager();

    public void RegisterCommands(){

        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            var rootBank = CommandManager.literal("bank").build();

            //second layer of tree
            var balance = CommandManager.literal("balance").build();
            var wire = CommandManager.literal("wire").build();

            //third layer of tree
            var setBalance = CommandManager.literal("setBalance").build();
            var getBalance = CommandManager.literal("getBalance").build();
            var addMoney = CommandManager.literal("addMoney").build();
            var subtractBalance = CommandManager.literal("subtractBalance").build();

            var playerArgAdd = CommandManager.argument("otherParty", EntityArgumentType.player()).build();
            var paymentArg = CommandManager.argument("payment", LongArgumentType.longArg()).build();

            var wireFunction = CommandManager.argument("reason", StringArgumentType.greedyString())
                                    .executes(context ->    Wire
                                                            (
                                                            EntityArgumentType.getPlayer(context, "otherParty"),
                                                            LongArgumentType.getLong(context, "payment"),
                                                            StringArgumentType.getString(context, "reason"),
                                                            context)
                                                            )
                                    .build();

            var setBalanceFunction = CommandManager.argument("reason", StringArgumentType.greedyString())
                                    .executes(context ->    SetBalance
                                                            (
                                                            EntityArgumentType.getPlayer(context, "otherParty"),
                                                            LongArgumentType.getLong(context, "payment"),
                                                            StringArgumentType.getString(context, "reason"),
                                                            context)
                                                            )
                                    .build();
            
            var addMoneyFunction = CommandManager.argument("reason", StringArgumentType.greedyString())
                                    .executes(context ->    AddMoney
                                                            (
                                                            EntityArgumentType.getPlayer(context, "otherParty"),
                                                            LongArgumentType.getLong(context, "payment"),
                                                            StringArgumentType.getString(context, "reason"),
                                                            context)
                                                            )
                                    .build();

            var subtractMoneyFunction = CommandManager.argument("reason", StringArgumentType.greedyString())
                                    .executes(context ->    SubtractBalance
                                                            (
                                                            EntityArgumentType.getPlayer(context, "otherParty"),
                                                            LongArgumentType.getLong(context, "payment"),
                                                            StringArgumentType.getString(context, "reason"),
                                                            context)
                                                            )
                                    .build();


            // Build out the argument tree here
            dispatcher.getRoot().addChild(rootBank);
            rootBank.addChild(wire);
            rootBank.addChild(balance);

            // Subcommands of balance node "/getbalance", [ADMIN->] "/setbalance", "/subtractbalance", "/addmoney"
            balance.addChild(getBalance);
            balance.addChild(setBalance);
            balance.addChild(subtractBalance);
            balance.addChild(addMoney);

            //subcommands of wire node "/playernamehere"
            wire.addChild(playerArgAdd);
            playerArgAdd.addChild(paymentArg);
            paymentArg.addChild(wireFunction);

            //subcommands of setBalance
            setBalance.addChild(playerArgAdd);
            playerArgAdd.addChild(paymentArg);
            paymentArg.addChild(setBalanceFunction);

            //subcommands of addMoney
            addMoney.addChild(playerArgAdd);
            playerArgAdd.addChild(paymentArg);
            paymentArg.addChild(addMoneyFunction);

            //subcommands for subtractbalance
            subtractBalance.addChild(playerArgAdd);
            playerArgAdd.addChild(paymentArg);
            paymentArg.addChild(subtractMoneyFunction);
        });
    }

    public Integer GetBalance(CommandContext<ServerCommandSource> context) {
        Integer ret = -1;
        
        if (context.getSource().isExecutedByPlayer()) {
            ServerPlayerEntity player = context.getSource().getPlayer();
            BankManager playerBank = new BankManager(player.getUuidAsString());
            player.sendMessage(Text.literal(String.valueOf(playerBank.GetBalance())));
            return 0;
        }
        
        return ret;
    }

    //SetBalance will be a ServerCommand only Perm level Op
    public Integer SetBalance(ServerPlayerEntity otherParty, long payment, String reason, CommandContext<ServerCommandSource> context) {
        Integer ret = -1;

        if (context.getSource().hasPermissionLevel(1)) {
            BankManager playerBank = new BankManager(otherParty.getUuidAsString());
            System.out.println("UUID TEST IN COMMANDS " + otherParty.getUuidAsString());
            playerBank.SetBalance(payment, reason, otherParty);
            ret = 0;
        }
        return ret;
    }

    //AddMoney will be a ServerCommand Only Perm level Op
    public Integer AddMoney(ServerPlayerEntity otherParty, long payment, String reason, CommandContext<ServerCommandSource> context) {
        Integer ret = -1;

        if (context.getSource().hasPermissionLevel(1)) {
            ret = 0;
            ServerPlayerEntity player = context.getSource().getPlayer();
            BankManager playerBank = new BankManager(player.getUuidAsString());
            playerBank.AddMoney(otherParty, payment, player, reason);
            player.sendMessage(Text.literal(String.valueOf(playerBank.GetBalance())));
        }

        return ret;
    }

    //SubtractBalance will be a ServerCommand Perm leve Op
    public Integer SubtractBalance(ServerPlayerEntity otherParty, long payment, String reason, CommandContext<ServerCommandSource> context) {
        Integer ret = -1;

        if (context.getSource().hasPermissionLevel(1)) {
            BankManager playerBank = new BankManager(otherParty.getUuidAsString());
            playerBank.SubtractBalance(reason, payment, otherParty);
            otherParty.sendMessage(Text.literal(String.valueOf(playerBank.GetBalance())));
            ret = 0;
        }
        return ret;
    }

    public Integer Wire(ServerPlayerEntity otherParty, long payment, String reason, CommandContext<ServerCommandSource> context) {
        Integer ret = -1;

        if (context.getSource().isExecutedByPlayer()) {
            ServerPlayerEntity sender = context.getSource().getPlayer();
            BankManager playerBank = new BankManager(sender.getUuidAsString());
            if (sender.getUuidAsString() == otherParty.getUuidAsString()) {
                sender.sendMessage(Text.of("you aint congress"));
                return -1;
            }
            playerBank.Wire(otherParty, payment, reason, sender);
            sender.sendMessage(Text.literal(String.valueOf(playerBank.GetBalance())));
            ret = 0;
        }

        return ret;
    }

}
